package org.apache.bcel.generic;

import java.io.DataOutputStream;
import java.io.IOException;
import org.apache.bcel.Const;
import org.apache.bcel.ExceptionConst;
import org.apache.bcel.util.ByteSequence;

public class NEWARRAY extends Instruction implements AllocationInstruction, ExceptionThrower, StackProducer {
   private byte type;

   NEWARRAY() {
   }

   public NEWARRAY(BasicType type) {
      this(type.getType());
   }

   public NEWARRAY(byte type) {
      super((short)188, (short)2);
      this.type = type;
   }

   public void accept(Visitor v) {
      v.visitAllocationInstruction(this);
      v.visitExceptionThrower(this);
      v.visitStackProducer(this);
      v.visitNEWARRAY(this);
   }

   public void dump(DataOutputStream out) throws IOException {
      out.writeByte(super.getOpcode());
      out.writeByte(this.type);
   }

   public Class<?>[] getExceptions() {
      return new Class[]{ExceptionConst.NEGATIVE_ARRAY_SIZE_EXCEPTION};
   }

   public final Type getType() {
      return new ArrayType(BasicType.getType(this.type), 1);
   }

   public final byte getTypecode() {
      return this.type;
   }

   protected void initFromFile(ByteSequence bytes, boolean wide) throws IOException {
      this.type = bytes.readByte();
      super.setLength(2);
   }

   public String toString(boolean verbose) {
      return super.toString(verbose) + " " + Const.getTypeName(this.type);
   }
}
